@@ -1,30 +1,14 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """The main statusbar widget."""
 
 import enum
 import dataclasses
 
-from PyQt5.QtCore import (pyqtSignal, pyqtSlot,  # type: ignore[attr-defined]
-                          pyqtProperty, Qt, QSize, QTimer)
-from PyQt5.QtWidgets import QWidget, QHBoxLayout, QStackedLayout, QSizePolicy
+from qutebrowser.qt.core import pyqtSignal, pyqtProperty, pyqtSlot, Qt, QSize, QTimer
+from qutebrowser.qt.widgets import QWidget, QHBoxLayout, QStackedLayout, QSizePolicy
 
 from qutebrowser.browser import browsertab
 from qutebrowser.config import config, stylesheet
@@ -32,7 +16,7 @@ from qutebrowser.keyinput import modeman
 from qutebrowser.utils import usertypes, log, objreg, utils
 from qutebrowser.mainwindow.statusbar import (backforward, command, progress,
                                               keystring, percentage, url,
-                                              tabindex, textbase)
+                                              tabindex, textbase, clock, searchmatch)
 
 
 @dataclasses.dataclass
@@ -144,6 +128,7 @@ class StatusBar(QWidget):
         url: The UrlText widget in the statusbar.
         prog: The Progress widget in the statusbar.
         cmd: The Command widget in the statusbar.
+        search_match: The SearchMatch widget in the statusbar.
         _hbox: The main QHBoxLayout.
         _stack: The QStackedLayout with cmd/txt widgets.
         _win_id: The window ID the statusbar is associated with.
@@ -165,10 +150,10 @@ class StatusBar(QWidget):
     def __init__(self, *, win_id, private, parent=None):
         super().__init__(parent)
         self.setObjectName(self.__class__.__name__)
-        self.setAttribute(Qt.WA_StyledBackground)
+        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground)
         stylesheet.set_register(self)
 
-        self.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Fixed)
+        self.setSizePolicy(QSizePolicy.Policy.Ignored, QSizePolicy.Policy.Fixed)
 
         self._win_id = win_id
         self._color_flags = ColorFlags()
@@ -194,12 +179,15 @@ class StatusBar(QWidget):
         self.cmd.hide_cmd.connect(self._hide_cmd_widget)
         self._hide_cmd_widget()
 
+        self.search_match = searchmatch.SearchMatch()
+
         self.url = url.UrlText()
         self.percentage = percentage.Percentage()
         self.backforward = backforward.Backforward()
         self.tabindex = tabindex.TabIndex()
         self.keystring = keystring.KeyString()
         self.prog = progress.Progress(self)
+        self.clock = clock.Clock()
         self._text_widgets = []
         self._draw_widgets()
 
@@ -209,6 +197,33 @@ class StatusBar(QWidget):
     def __repr__(self):
         return utils.get_repr(self)
 
+    def _get_widget_from_config(self, key):
+        """Return the widget that fits with config string key."""
+        if key == 'url':
+            return self.url
+        elif key == 'scroll':
+            return self.percentage
+        elif key == 'scroll_raw':
+            return self.percentage
+        elif key == 'history':
+            return self.backforward
+        elif key == 'tabs':
+            return self.tabindex
+        elif key == 'keypress':
+            return self.keystring
+        elif key == 'progress':
+            return self.prog
+        elif key == 'search_match':
+            return self.search_match
+        elif key.startswith('text:'):
+            new_text_widget = textbase.TextBase()
+            self._text_widgets.append(new_text_widget)
+            return new_text_widget
+        elif key.startswith('clock:') or key == 'clock':
+            return self.clock
+        else:
+            raise utils.Unreachable(key)
+
     @pyqtSlot(str)
     def _on_config_changed(self, option):
         if option == 'statusbar.show':
@@ -226,48 +241,39 @@ class StatusBar(QWidget):
 
         # Read the list and set widgets accordingly
         for segment in config.val.statusbar.widgets:
-            if segment == 'url':
-                self._hbox.addWidget(self.url)
-                self.url.show()
-            elif segment == 'scroll':
-                self._hbox.addWidget(self.percentage)
-                self.percentage.show()
-            elif segment == 'scroll_raw':
-                self._hbox.addWidget(self.percentage)
-                self.percentage.set_raw()
-                self.percentage.show()
-            elif segment == 'history':
-                self._hbox.addWidget(self.backforward)
-                self.backforward.enabled = True
+            widget = self._get_widget_from_config(segment)
+            self._hbox.addWidget(widget)
+
+            if segment == 'scroll_raw':
+                widget.set_raw()
+            elif segment in ('history', 'progress'):
+                widget.enabled = True
                 if tab:
-                    self.backforward.on_tab_changed(tab)
-            elif segment == 'tabs':
-                self._hbox.addWidget(self.tabindex)
-                self.tabindex.show()
-            elif segment == 'keypress':
-                self._hbox.addWidget(self.keystring)
-                self.keystring.show()
-            elif segment == 'progress':
-                self._hbox.addWidget(self.prog)
-                self.prog.enabled = True
-                if tab:
-                    self.prog.on_tab_changed(tab)
+                    widget.on_tab_changed(tab)
+
+                # Do not call .show() for these widgets. They are not always shown, and
+                # dynamically show/hide themselves in their on_tab_changed() methods.
+                continue
             elif segment.startswith('text:'):
-                cur_widget = textbase.TextBase()
-                self._text_widgets.append(cur_widget)
-                cur_widget.setText(segment.split(':', maxsplit=1)[1])
-                self._hbox.addWidget(cur_widget)
-                cur_widget.show()
-            else:
-                raise utils.Unreachable(segment)
+                widget.setText(segment.split(':', maxsplit=1)[1])
+            elif segment.startswith('clock:') or segment == 'clock':
+                split_segment = segment.split(':', maxsplit=1)
+                if len(split_segment) == 2 and split_segment[1]:
+                    widget.format = split_segment[1]
+                else:
+                    widget.format = '%X'
+
+            widget.show()
 
     def _clear_widgets(self):
         """Clear widgets before redrawing them."""
         # Start with widgets hidden and show them when needed
         for widget in [self.url, self.percentage,
                        self.backforward, self.tabindex,
-                       self.keystring, self.prog, *self._text_widgets]:
+                       self.keystring, self.prog, self.clock, *self._text_widgets]:
             assert isinstance(widget, QWidget)
+            if widget in [self.prog, self.backforward]:
+                widget.enabled = False  # type: ignore[attr-defined]
             widget.hide()
             self._hbox.removeWidget(widget)
         self._text_widgets.clear()
@@ -300,7 +306,7 @@ class StatusBar(QWidget):
         padding = config.val.statusbar.padding
         self._hbox.setContentsMargins(padding.left, 0, padding.right, 0)
 
-    @pyqtProperty('QStringList')
+    @pyqtProperty('QStringList')  # type: ignore[type-var]
     def color_flags(self):
         """Getter for self.color_flags, so it can be used as Qt property."""
         return self._color_flags.to_stringlist()
