@@ -1,20 +1,7 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
 
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.config.configcommands."""
 
@@ -23,7 +10,7 @@ import functools
 import unittest.mock
 
 import pytest
-from PyQt5.QtCore import QUrl
+from qutebrowser.qt.core import QUrl
 
 from qutebrowser.config import configcommands
 from qutebrowser.api import cmdutils
@@ -212,12 +199,17 @@ class TestSet:
             commands.set(win_id=0, option='foo?')
 
 
-def test_diff(commands, tabbed_browser_stubs):
+@pytest.mark.parametrize("include_hidden, url", [
+    (True, "qute://configdiff?include_hidden=true"),
+    (False, "qute://configdiff"),
+])
+def test_diff(commands, tabbed_browser_stubs, include_hidden, url):
     """Run ':config-diff'.
 
-    Should open qute://configdiff."""
-    commands.config_diff(win_id=0)
-    assert tabbed_browser_stubs[0].loaded_url == QUrl('qute://configdiff')
+    Should open qute://configdiff.
+    """
+    commands.config_diff(win_id=0, include_hidden=include_hidden)
+    assert tabbed_browser_stubs[0].loaded_url == QUrl(url)
 
 
 class TestCycle:
@@ -373,6 +365,15 @@ class TestAdd:
         with pytest.raises(cmdutils.CommandError, match="Invalid value ''"):
             commands.config_dict_add('aliases', 'missingkey', '')
 
+    def test_dict_add_value_type(self, commands, config_stub):
+        commands.config_dict_add(
+            "content.javascript.log_message.levels",
+            "example",
+            "['error']",
+        )
+        value = config_stub.val.content.javascript.log_message.levels["example"]
+        assert value == ['error']
+
 
 class TestRemove:
 
@@ -508,13 +509,12 @@ class TestUnsetAndClear:
         assert yaml_value(name) == (usertypes.UNSET if save else 'never')
 
 
+@pytest.mark.usefixtures('config_tmpdir', 'data_tmpdir',
+                         'config_stub', 'key_config_stub')
 class TestSource:
 
     """Test :config-source."""
 
-    pytestmark = pytest.mark.usefixtures('config_tmpdir', 'data_tmpdir',
-                                         'config_stub', 'key_config_stub')
-
     @pytest.mark.parametrize('location', ['default', 'absolute', 'relative'])
     @pytest.mark.parametrize('clear', [True, False])
     def test_config_source(self, tmp_path, commands, config_stub, config_tmpdir,
@@ -592,14 +592,13 @@ class TestSource:
         assert str(excinfo.value) == expected
 
 
+@pytest.mark.usefixtures('config_tmpdir', 'data_tmpdir',
+                         'config_stub', 'key_config_stub',
+                         'qapp')
 class TestEdit:
 
     """Tests for :config-edit."""
 
-    pytestmark = pytest.mark.usefixtures('config_tmpdir', 'data_tmpdir',
-                                         'config_stub', 'key_config_stub',
-                                         'qapp')
-
     def test_no_source(self, commands, mocker):
         mock = mocker.patch('qutebrowser.config.configcommands.editor.'
                             'ExternalEditor._start_editor', autospec=True)
