@@ -1,27 +1,12 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>:
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>:
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Tests for BaseKeyParser."""
 
 from unittest import mock
 
-from PyQt5.QtCore import Qt
+from qutebrowser.qt.core import Qt
 import pytest
 
 from qutebrowser.keyinput import basekeyparser, keyutils
@@ -54,7 +39,7 @@ def handle_text():
     """Helper function to handle multiple fake keypresses."""
     def func(kp, *args):
         for key in args:
-            info = keyutils.KeyInfo(key, Qt.NoModifier)
+            info = keyutils.KeyInfo(key, Qt.KeyboardModifier.NoModifier)
             kp.handle(info.to_event())
     return func
 
@@ -119,11 +104,11 @@ def test_read_config(keyparser, key_config_stub, changed_mode, expected):
 class TestHandle:
 
     def test_valid_key(self, prompt_keyparser, handle_text):
-        modifier = Qt.MetaModifier if utils.is_mac else Qt.ControlModifier
+        modifier = Qt.KeyboardModifier.MetaModifier if utils.is_mac else Qt.KeyboardModifier.ControlModifier
 
         infos = [
-            keyutils.KeyInfo(Qt.Key_A, modifier),
-            keyutils.KeyInfo(Qt.Key_X, modifier),
+            keyutils.KeyInfo(Qt.Key.Key_A, modifier),
+            keyutils.KeyInfo(Qt.Key.Key_X, modifier),
         ]
         for info in infos:
             prompt_keyparser.handle(info.to_event())
@@ -133,11 +118,11 @@ class TestHandle:
         assert not prompt_keyparser._sequence
 
     def test_valid_key_count(self, prompt_keyparser):
-        modifier = Qt.MetaModifier if utils.is_mac else Qt.ControlModifier
+        modifier = Qt.KeyboardModifier.MetaModifier if utils.is_mac else Qt.KeyboardModifier.ControlModifier
 
         infos = [
-            keyutils.KeyInfo(Qt.Key_5, Qt.NoModifier),
-            keyutils.KeyInfo(Qt.Key_A, modifier),
+            keyutils.KeyInfo(Qt.Key.Key_5, Qt.KeyboardModifier.NoModifier),
+            keyutils.KeyInfo(Qt.Key.Key_A, modifier),
         ]
         for info in infos:
             prompt_keyparser.handle(info.to_event())
@@ -145,10 +130,10 @@ class TestHandle:
             'message-info ctrla', 5)
 
     @pytest.mark.parametrize('keys', [
-        [(Qt.Key_B, Qt.NoModifier), (Qt.Key_C, Qt.NoModifier)],
-        [(Qt.Key_A, Qt.ControlModifier | Qt.AltModifier)],
+        [(Qt.Key.Key_B, Qt.KeyboardModifier.NoModifier), (Qt.Key.Key_C, Qt.KeyboardModifier.NoModifier)],
+        [(Qt.Key.Key_A, Qt.KeyboardModifier.ControlModifier | Qt.KeyboardModifier.AltModifier)],
         # Only modifier
-        [(Qt.Key_Shift, Qt.ShiftModifier)],
+        [(Qt.Key.Key_Shift, Qt.KeyboardModifier.ShiftModifier)],
     ])
     def test_invalid_keys(self, prompt_keyparser, keys):
         for key, modifiers in keys:
@@ -158,40 +143,40 @@ class TestHandle:
         assert not prompt_keyparser._sequence
 
     def test_dry_run(self, prompt_keyparser):
-        b_info = keyutils.KeyInfo(Qt.Key_B, Qt.NoModifier)
+        b_info = keyutils.KeyInfo(Qt.Key.Key_B, Qt.KeyboardModifier.NoModifier)
         prompt_keyparser.handle(b_info.to_event())
 
-        a_info = keyutils.KeyInfo(Qt.Key_A, Qt.NoModifier)
+        a_info = keyutils.KeyInfo(Qt.Key.Key_A, Qt.KeyboardModifier.NoModifier)
         prompt_keyparser.handle(a_info.to_event(), dry_run=True)
 
         assert not prompt_keyparser.execute.called
         assert prompt_keyparser._sequence
 
     def test_dry_run_count(self, prompt_keyparser):
-        info = keyutils.KeyInfo(Qt.Key_9, Qt.NoModifier)
+        info = keyutils.KeyInfo(Qt.Key.Key_9, Qt.KeyboardModifier.NoModifier)
         prompt_keyparser.handle(info.to_event(), dry_run=True)
         assert not prompt_keyparser._count
 
     def test_invalid_key(self, prompt_keyparser):
-        keys = [Qt.Key_B, 0x0]
+        keys = [Qt.Key.Key_B, keyutils._NIL_KEY]
         for key in keys:
-            info = keyutils.KeyInfo(key, Qt.NoModifier)
+            info = keyutils.KeyInfo(key, Qt.KeyboardModifier.NoModifier)
             prompt_keyparser.handle(info.to_event())
         assert not prompt_keyparser._sequence
 
     def test_valid_keychain(self, handle_text, prompt_keyparser):
         handle_text(prompt_keyparser,
                     # Press 'x' which is ignored because of no match
-                    Qt.Key_X,
+                    Qt.Key.Key_X,
                     # Then start the real chain
-                    Qt.Key_B, Qt.Key_A)
+                    Qt.Key.Key_B, Qt.Key.Key_A)
         prompt_keyparser.execute.assert_called_with('message-info ba', None)
         assert not prompt_keyparser._sequence
 
     @pytest.mark.parametrize('key, modifiers, number', [
-        (Qt.Key_0, Qt.NoModifier, 0),
-        (Qt.Key_1, Qt.NoModifier, 1),
-        (Qt.Key_1, Qt.KeypadModifier, 1),
+        (Qt.Key.Key_0, Qt.KeyboardModifier.NoModifier, 0),
+        (Qt.Key.Key_1, Qt.KeyboardModifier.NoModifier, 1),
+        (Qt.Key.Key_1, Qt.KeyboardModifier.KeypadModifier, 1),
     ])
     def test_number_press(self, prompt_keyparser,
                           key, modifiers, number):
@@ -201,8 +186,8 @@ class TestHandle:
         assert not prompt_keyparser._sequence
 
     @pytest.mark.parametrize('modifiers, text', [
-        (Qt.NoModifier, '2'),
-        (Qt.KeypadModifier, 'num-2'),
+        (Qt.KeyboardModifier.NoModifier, '2'),
+        (Qt.KeyboardModifier.KeypadModifier, 'num-2'),
     ])
     def test_number_press_keypad(self, keyparser, config_stub,
                                  modifiers, text):
@@ -210,18 +195,18 @@ class TestHandle:
         config_stub.val.bindings.commands = {'normal': {
             '2': 'message-info 2',
             '<Num+2>': 'message-info num-2'}}
-        keyparser.handle(keyutils.KeyInfo(Qt.Key_2, modifiers).to_event())
+        keyparser.handle(keyutils.KeyInfo(Qt.Key.Key_2, modifiers).to_event())
         command = 'message-info {}'.format(text)
         keyparser.execute.assert_called_once_with(command, None)
         assert not keyparser._sequence
 
     def test_umlauts(self, handle_text, keyparser, config_stub):
         config_stub.val.bindings.commands = {'normal': {'ü': 'message-info ü'}}
-        handle_text(keyparser, Qt.Key_Udiaeresis)
+        handle_text(keyparser, Qt.Key.Key_Udiaeresis)
         keyparser.execute.assert_called_once_with('message-info ü', None)
 
     def test_mapping(self, config_stub, handle_text, prompt_keyparser):
-        handle_text(prompt_keyparser, Qt.Key_X)
+        handle_text(prompt_keyparser, Qt.Key.Key_X)
         prompt_keyparser.execute.assert_called_once_with(
             'message-info a', None)
 
@@ -230,27 +215,27 @@ class TestHandle:
         config_stub.val.bindings.commands = {'normal': {'a': 'nop'}}
         config_stub.val.bindings.key_mappings = {'1': 'a'}
 
-        info = keyutils.KeyInfo(Qt.Key_1, Qt.KeypadModifier)
+        info = keyutils.KeyInfo(Qt.Key.Key_1, Qt.KeyboardModifier.KeypadModifier)
         keyparser.handle(info.to_event())
         keyparser.execute.assert_called_once_with('nop', None)
 
     def test_binding_and_mapping(self, config_stub, handle_text, prompt_keyparser):
         """with a conflicting binding/mapping, the binding should win."""
-        handle_text(prompt_keyparser, Qt.Key_B)
+        handle_text(prompt_keyparser, Qt.Key.Key_B)
         assert not prompt_keyparser.execute.called
 
     def test_mapping_in_key_chain(self, config_stub, handle_text, keyparser):
         """A mapping should work even as part of a keychain."""
         config_stub.val.bindings.commands = {'normal':
                                              {'aa': 'message-info aa'}}
-        handle_text(keyparser, Qt.Key_A, Qt.Key_X)
+        handle_text(keyparser, Qt.Key.Key_A, Qt.Key.Key_X)
         keyparser.execute.assert_called_once_with('message-info aa', None)
 
     def test_binding_with_shift(self, prompt_keyparser):
         """Simulate a binding which involves shift."""
-        for key, modifiers in [(Qt.Key_Y, Qt.NoModifier),
-                               (Qt.Key_Shift, Qt.ShiftModifier),
-                               (Qt.Key_Y, Qt.ShiftModifier)]:
+        for key, modifiers in [(Qt.Key.Key_Y, Qt.KeyboardModifier.NoModifier),
+                               (Qt.Key.Key_Shift, Qt.KeyboardModifier.ShiftModifier),
+                               (Qt.Key.Key_Y, Qt.KeyboardModifier.ShiftModifier)]:
             info = keyutils.KeyInfo(key, modifiers)
             prompt_keyparser.handle(info.to_event())
 
@@ -264,7 +249,7 @@ class TestHandle:
                 'a': 'message-info foo'
             }
         }
-        info = keyutils.KeyInfo(Qt.Key_A, Qt.NoModifier)
+        info = keyutils.KeyInfo(Qt.Key.Key_A, Qt.KeyboardModifier.NoModifier)
         keyparser.handle(info.to_event())
         keyparser.execute.assert_called_once_with('message-info foo', None)
 
@@ -275,39 +260,39 @@ class TestCount:
 
     def test_no_count(self, handle_text, prompt_keyparser):
         """Test with no count added."""
-        handle_text(prompt_keyparser, Qt.Key_B, Qt.Key_A)
+        handle_text(prompt_keyparser, Qt.Key.Key_B, Qt.Key.Key_A)
         prompt_keyparser.execute.assert_called_once_with(
             'message-info ba', None)
         assert not prompt_keyparser._sequence
 
     def test_count_0(self, handle_text, prompt_keyparser):
-        handle_text(prompt_keyparser, Qt.Key_0, Qt.Key_B, Qt.Key_A)
+        handle_text(prompt_keyparser, Qt.Key.Key_0, Qt.Key.Key_B, Qt.Key.Key_A)
         calls = [mock.call('message-info 0', None),
                  mock.call('message-info ba', None)]
         prompt_keyparser.execute.assert_has_calls(calls)
         assert not prompt_keyparser._sequence
 
     def test_count_42(self, handle_text, prompt_keyparser):
-        handle_text(prompt_keyparser, Qt.Key_4, Qt.Key_2, Qt.Key_B, Qt.Key_A)
+        handle_text(prompt_keyparser, Qt.Key.Key_4, Qt.Key.Key_2, Qt.Key.Key_B, Qt.Key.Key_A)
         prompt_keyparser.execute.assert_called_once_with('message-info ba', 42)
         assert not prompt_keyparser._sequence
 
     def test_count_42_invalid(self, handle_text, prompt_keyparser):
         # Invalid call with ccx gets ignored
         handle_text(prompt_keyparser,
-                    Qt.Key_4, Qt.Key_2, Qt.Key_C, Qt.Key_C, Qt.Key_X)
+                    Qt.Key.Key_4, Qt.Key.Key_2, Qt.Key.Key_C, Qt.Key.Key_C, Qt.Key.Key_X)
         assert not prompt_keyparser.execute.called
         assert not prompt_keyparser._sequence
         # Valid call with ccc gets the correct count
         handle_text(prompt_keyparser,
-                    Qt.Key_2, Qt.Key_3, Qt.Key_C, Qt.Key_C, Qt.Key_C)
+                    Qt.Key.Key_2, Qt.Key.Key_3, Qt.Key.Key_C, Qt.Key.Key_C, Qt.Key.Key_C)
         prompt_keyparser.execute.assert_called_once_with(
             'message-info ccc', 23)
         assert not prompt_keyparser._sequence
 
     def test_superscript(self, handle_text, prompt_keyparser):
         # https://github.com/qutebrowser/qutebrowser/issues/3743
-        handle_text(prompt_keyparser, Qt.Key_twosuperior, Qt.Key_B, Qt.Key_A)
+        handle_text(prompt_keyparser, Qt.Key.Key_twosuperior, Qt.Key.Key_B, Qt.Key.Key_A)
 
     def test_count_keystring_update(self, qtbot,
                                     handle_text, prompt_keyparser):
@@ -315,17 +300,17 @@ class TestCount:
         with qtbot.wait_signals([
                 prompt_keyparser.keystring_updated,
                 prompt_keyparser.keystring_updated]) as blocker:
-            handle_text(prompt_keyparser, Qt.Key_4, Qt.Key_2)
+            handle_text(prompt_keyparser, Qt.Key.Key_4, Qt.Key.Key_2)
         sig1, sig2 = blocker.all_signals_and_args
         assert sig1.args == ('4',)
         assert sig2.args == ('42',)
 
     def test_numpad(self, prompt_keyparser):
         """Make sure we can enter a count via numpad."""
-        for key, modifiers in [(Qt.Key_4, Qt.KeypadModifier),
-                               (Qt.Key_2, Qt.KeypadModifier),
-                               (Qt.Key_B, Qt.NoModifier),
-                               (Qt.Key_A, Qt.NoModifier)]:
+        for key, modifiers in [(Qt.Key.Key_4, Qt.KeyboardModifier.KeypadModifier),
+                               (Qt.Key.Key_2, Qt.KeyboardModifier.KeypadModifier),
+                               (Qt.Key.Key_B, Qt.KeyboardModifier.NoModifier),
+                               (Qt.Key.Key_A, Qt.KeyboardModifier.NoModifier)]:
             info = keyutils.KeyInfo(key, modifiers)
             prompt_keyparser.handle(info.to_event())
         prompt_keyparser.execute.assert_called_once_with('message-info ba', 42)
@@ -352,7 +337,7 @@ def test_respect_config_when_matching_counts(keyparser, config_stub):
     """Don't match counts if disabled in the config."""
     config_stub.val.input.match_counts = False
 
-    info = keyutils.KeyInfo(Qt.Key_1, Qt.NoModifier)
+    info = keyutils.KeyInfo(Qt.Key.Key_1, Qt.KeyboardModifier.NoModifier)
     keyparser.handle(info.to_event())
 
     assert not keyparser._sequence
