@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Tests for qutebrowser.utils.version."""
 
@@ -29,11 +14,13 @@ import logging
 import textwrap
 import datetime
 import dataclasses
+import importlib.metadata
 
 import pytest
 import hypothesis
 import hypothesis.strategies
-from PyQt5.QtCore import PYQT_VERSION_STR
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import PYQT_VERSION_STR
 
 import qutebrowser
 from qutebrowser.config import config, websettings
@@ -657,8 +644,8 @@ class TestModuleVersions:
         assert version._module_versions() == expected
 
     @pytest.mark.parametrize('module, idx, expected', [
-        ('colorama', 1, 'colorama: no'),
-        ('adblock', 5, 'adblock: no'),
+        ('colorama', 0, 'colorama: no'),
+        ('adblock', 4, 'adblock: no'),
     ])
     def test_missing_module(self, module, idx, expected, import_fake):
         """Test with a module missing.
@@ -706,11 +693,11 @@ class TestModuleVersions:
         assert not mod_info.is_usable()
 
         expected = f"adblock: {fake_version} (< {mod_info.min_version}, outdated)"
-        assert version._module_versions()[5] == expected
+        assert version._module_versions()[4] == expected
 
     @pytest.mark.parametrize('attribute, expected_modules', [
         ('VERSION', ['colorama']),
-        ('SIP_VERSION_STR', ['sip']),
+        ('SIP_VERSION_STR', ['PyQt5.sip', 'PyQt6.sip']),
         (None, []),
     ])
     def test_version_attribute(self, attribute, expected_modules, import_fake):
@@ -751,6 +738,7 @@ class TestModuleVersions:
         ('adblock', True),
         ('dataclasses', False),
         ('importlib_resources', False),
+        ('objc', True),
     ])
     def test_existing_attributes(self, name, has_version):
         """Check if all dependencies have an expected __version__ attribute.
@@ -978,13 +966,12 @@ class TestWebEngineVersions:
         assert version.WebEngineVersions.from_elf(elf_version) == expected
 
     @pytest.mark.parametrize('pyqt_version, chromium_version', [
-        ('5.12.10', '69.0.3497.128'),
-        ('5.14.2', '77.0.3865.129'),
-        ('5.15.1', '80.0.3987.163'),
         ('5.15.2', '83.0.4103.122'),
         ('5.15.3', '87.0.4280.144'),
         ('5.15.4', '87.0.4280.144'),
         ('5.15.5', '87.0.4280.144'),
+        ('6.2.0', '90.0.4430.228'),
+        ('6.3.0', '94.0.4606.126'),
     ])
     def test_from_pyqt(self, freezer, pyqt_version, chromium_version):
         if freezer and pyqt_version in ['5.15.3', '5.15.4', '5.15.5']:
@@ -1005,16 +992,16 @@ class TestWebEngineVersions:
         pyqt_webengine_version = version._get_pyqt_webengine_qt_version()
         if pyqt_webengine_version is None:
             if '.dev' in PYQT_VERSION_STR:
-                pytest.skip("dev version of PyQt5")
+                pytest.skip("dev version of PyQt")
 
             try:
-                from PyQt5.QtWebEngine import (
+                from qutebrowser.qt.webenginecore import (
                     PYQT_WEBENGINE_VERSION_STR, PYQT_WEBENGINE_VERSION)
             except ImportError as e:
-                # QtWebKit or QtWebEngine < 5.13
+                # QtWebKit
                 pytest.skip(str(e))
 
-            if PYQT_WEBENGINE_VERSION >= 0x050F02:
+            if 0x060000 > PYQT_WEBENGINE_VERSION >= 0x050F02:
                 # Starting with Qt 5.15.2, we can only do bad guessing anyways...
                 pytest.skip("Could be QtWebEngine 5.15.2 or 5.15.3")
 
@@ -1062,18 +1049,18 @@ class TestChromiumVersion:
 
     @pytest.fixture(autouse=True)
     def clear_parsed_ua(self, monkeypatch):
-        pytest.importorskip('PyQt5.QtWebEngineWidgets')
+        pytest.importorskip('qutebrowser.qt.webenginewidgets')
         if webenginesettings is not None:
             # Not available with QtWebKit
             monkeypatch.setattr(webenginesettings, 'parsed_user_agent', None)
 
-    def test_fake_ua(self, monkeypatch, caplog):
+    def test_fake_ua(self, monkeypatch, caplog, patch_no_api):
         ver = '77.0.3865.98'
         webenginesettings._init_user_agent_str(_QTWE_USER_AGENT.format(ver))
 
         assert version.qtwebengine_versions().chromium == ver
 
-    def test_prefers_saved_user_agent(self, monkeypatch):
+    def test_prefers_saved_user_agent(self, monkeypatch, patch_no_api):
         webenginesettings._init_user_agent_str(_QTWE_USER_AGENT.format('87'))
 
         class FakeProfile:
@@ -1089,64 +1076,53 @@ class TestChromiumVersion:
 
     def test_avoided(self, monkeypatch):
         versions = version.qtwebengine_versions(avoid_init=True)
-        assert versions.source in ['ELF', 'importlib', 'PyQt', 'Qt']
+        assert versions.source in ['api', 'ELF', 'importlib', 'PyQt', 'Qt']
+
+    @pytest.fixture
+    def patch_no_api(self, monkeypatch):
+        """Simulate no PyQt API for getting its version."""
+        monkeypatch.delattr(
+            qutebrowser.qt.webenginecore,
+            "qWebEngineVersion",
+            raising=False,
+        )
 
     @pytest.fixture
     def patch_elf_fail(self, monkeypatch):
         """Simulate parsing the version from ELF to fail."""
         monkeypatch.setattr(elf, 'parse_webenginecore', lambda: None)
 
-    @pytest.fixture
-    def patch_old_pyqt(self, monkeypatch):
-        """Simulate an old PyQt without PYQT_WEBENGINE_VERSION_STR."""
-        monkeypatch.setattr(version, 'PYQT_WEBENGINE_VERSION_STR', None)
-
-    @pytest.fixture
-    def patch_no_importlib(self, monkeypatch, stubs):
-        """Simulate missing importlib modules."""
-        import_fake = stubs.ImportFake({
-            'importlib_metadata': False,
-            'importlib.metadata': False,
-        }, monkeypatch)
-        import_fake.patch()
-
     @pytest.fixture
     def importlib_patcher(self, monkeypatch):
         """Patch the importlib module."""
-        def _patch(*, qt, qt5):
-            try:
-                import importlib.metadata as importlib_metadata
-            except ImportError:
-                importlib_metadata = pytest.importorskip("importlib_metadata")
-
+        def _patch(*, qt, qt5, qt6):
             def _fake_version(name):
                 if name == 'PyQtWebEngine-Qt':
                     outcome = qt
                 elif name == 'PyQtWebEngine-Qt5':
                     outcome = qt5
+                elif name == 'PyQt6-WebEngine-Qt6':
+                    outcome = qt6
                 else:
-                    raise utils.Unreachable(outcome)
+                    raise utils.Unreachable(name)
 
                 if outcome is None:
-                    raise importlib_metadata.PackageNotFoundError(name)
+                    raise importlib.metadata.PackageNotFoundError(name)
                 return outcome
 
-            monkeypatch.setattr(importlib_metadata, 'version', _fake_version)
+            monkeypatch.setattr(importlib.metadata, 'version', _fake_version)
 
         return _patch
 
     @pytest.fixture
     def patch_importlib_no_package(self, importlib_patcher):
-        """Simulate importlib not finding PyQtWebEngine-Qt[5]."""
-        importlib_patcher(qt=None, qt5=None)
+        """Simulate importlib not finding PyQtWebEngine Qt packages."""
+        importlib_patcher(qt=None, qt5=None, qt6=None)
 
     @pytest.mark.parametrize('patches, sources', [
-        (['elf_fail'], ['importlib', 'PyQt', 'Qt']),
-        (['elf_fail', 'old_pyqt'], ['importlib', 'Qt']),
-        (['elf_fail', 'no_importlib'], ['PyQt', 'Qt']),
-        (['elf_fail', 'no_importlib', 'old_pyqt'], ['Qt']),
-        (['elf_fail', 'importlib_no_package'], ['PyQt', 'Qt']),
-        (['elf_fail', 'importlib_no_package', 'old_pyqt'], ['Qt']),
+        (['no_api'], ['ELF', 'importlib', 'PyQt', 'Qt']),
+        (['no_api', 'elf_fail'], ['importlib', 'PyQt', 'Qt']),
+        (['no_api', 'elf_fail', 'importlib_no_package'], ['PyQt', 'Qt']),
     ], ids=','.join)
     def test_simulated(self, request, patches, sources):
         """Test various simulated error conditions.
@@ -1162,17 +1138,41 @@ class TestChromiumVersion:
         versions = version.qtwebengine_versions(avoid_init=True)
         assert versions.source in sources
 
-    @pytest.mark.parametrize('qt, qt5, expected', [
-        (None, '5.15.4', utils.VersionNumber(5, 15, 4)),
-        ('5.15.3', None, utils.VersionNumber(5, 15, 3)),
-        ('5.15.3', '5.15.4', utils.VersionNumber(5, 15, 4)),  # -Qt5 takes precedence
+    @pytest.mark.parametrize('qt, qt5, qt6, expected', [
+        pytest.param(
+            None, None, '6.3.0',
+            utils.VersionNumber(6, 3),
+            marks=pytest.mark.qt6_only,
+        ),
+        pytest.param(
+            '5.15.3', '5.15.4', '6.3.0',
+            utils.VersionNumber(6, 3),
+            marks=pytest.mark.qt6_only,
+        ),
+
+        pytest.param(
+            None, '5.15.4', None,
+            utils.VersionNumber(5, 15, 4),
+            marks=pytest.mark.qt5_only,
+        ),
+        pytest.param(
+            '5.15.3', None, None,
+            utils.VersionNumber(5, 15, 3),
+            marks=pytest.mark.qt5_only,
+        ),
+        # -Qt5 takes precedence
+        pytest.param(
+            '5.15.3', '5.15.4', None,
+            utils.VersionNumber(5, 15, 4),
+            marks=pytest.mark.qt5_only,
+        ),
     ])
-    def test_importlib(self, qt, qt5, expected, patch_elf_fail, importlib_patcher):
+    def test_importlib(self, qt, qt5, qt6, expected, patch_elf_fail, patch_no_api, importlib_patcher):
         """Test the importlib version logic with different Qt packages.
 
         With PyQtWebEngine 5.15.4, PyQtWebEngine-Qt was renamed to PyQtWebEngine-Qt5.
         """
-        importlib_patcher(qt=qt, qt5=qt5)
+        importlib_patcher(qt=qt, qt5=qt5, qt6=qt6)
         versions = version.qtwebengine_versions(avoid_init=True)
         assert versions.source == 'importlib'
         assert versions.webengine == expected
@@ -1181,9 +1181,10 @@ class TestChromiumVersion:
         utils.VersionNumber(5, 12, 10),
         utils.VersionNumber(5, 15, 3),
     ])
-    def test_override(self, monkeypatch, override):
+    @pytest.mark.parametrize('avoid_init', [True, False])
+    def test_override(self, monkeypatch, override, avoid_init):
         monkeypatch.setenv('QUTE_QTWEBENGINE_VERSION_OVERRIDE', str(override))
-        versions = version.qtwebengine_versions(avoid_init=True)
+        versions = version.qtwebengine_versions(avoid_init=avoid_init)
         assert versions.source == 'override'
         assert versions.webengine == override
 
@@ -1236,10 +1237,14 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
         '_path_info': lambda: {'PATH DESC': 'PATH NAME'},
         'objects.qapp': (stubs.FakeQApplication(style='STYLE', platform_name='PLATFORM')
                          if params.qapp else None),
-        'QLibraryInfo.location': (lambda _loc: 'QT PATH'),
+        'qtutils.library_path': (lambda _loc: 'QT PATH'),
         'sql.version': lambda: 'SQLITE VERSION',
         '_uptime': lambda: datetime.timedelta(hours=1, minutes=23, seconds=45),
         'config.instance.yaml_loaded': params.autoconfig_loaded,
+        'machinery.INFO': machinery.SelectionInfo(
+            wrapper="QT WRAPPER",
+            reason=machinery.SelectionReason.fake
+        ),
     }
 
     version.opengl_info.cache_clear()
@@ -1311,6 +1316,8 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
         PYTHON IMPLEMENTATION: PYTHON VERSION
         PyQt: PYQT VERSION
 
+        Qt wrapper: QT WRAPPER (via fake)
+
         MODULE VERSION 1
         MODULE VERSION 2
         pdf.js: PDFJS VERSION
@@ -1344,7 +1351,7 @@ class TestOpenGLInfo:
 
     def test_func(self, qapp):
         """Simply call version.opengl_info() and see if it doesn't crash."""
-        pytest.importorskip("PyQt5.QtOpenGL")
+        pytest.importorskip("qutebrowser.qt.opengl")
         version.opengl_info()
 
     def test_func_fake(self, qapp, monkeypatch):
@@ -1463,7 +1470,11 @@ def test_uptime(monkeypatch, qapp):
     monkeypatch.setattr(qapp, "launch_time", launch_time, raising=False)
 
     class FakeDateTime(datetime.datetime):
-        now = lambda x=datetime.datetime(1, 1, 1, 1, 1, 1, 2): x
+
+        @classmethod
+        def now(cls, tz=None):
+            return datetime.datetime(1, 1, 1, 1, 1, 1, 2)
+
     monkeypatch.setattr(datetime, 'datetime', FakeDateTime)
 
     uptime_delta = version._uptime()
